from ConfigParser import NoSectionError
from twindb_infrastructure.config import TWINDB_INFRA_CONFIG
from twindb_infrastructure.config.credentials import Credentials


class CloudFlareCredentials(Credentials):

    cloudflare_email = None
    cloudflare_auth_key = None

    def __init__(self, config_path=TWINDB_INFRA_CONFIG):
        super(CloudFlareCredentials, self).__init__(config_path=config_path)
        try:
            for option in self.config.options('cloudflare'):
                value = self.config.get('cloudflare', option).strip('"\'')
                setattr(self, option, value)

        except NoSectionError:
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_twindb_infrastructure
----------------------------------

Tests for `twindb_infrastructure` module.
"""
import logging

from click.testing import CliRunner
import mock
import pytest

from twindb_infrastructure import cli, setup_logging
from twindb_infrastructure.util import printf


def test_command_line_interface():
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert 'twindb_infrastructure.cli.main' in result.output

    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help  Show this message and exit.' in help_result.output


@mock.patch('twindb_infrastructure.util.sys')
def test_printf(mock_sys):
    printf('foo')
    mock_sys.stdout.write.assert_called_once_with('foo')


@pytest.mark.parametrize('debug,level', [
    (
        False,
        logging.INFO
    ),
    (
        True,
        logging.DEBUG
    ),
])
def test_setup_logging(debug, level):
    logger = logging.getLogger(__name__)
    setup_logging(logger, debug=debug)
    assert logger.level == level



import json
from subprocess import Popen, PIPE
from twindb_infrastructure import log

CLOUDFLARE_EMAIL = ""
CLOUDFLARE_AUTH_KEY = ""


def cf_api_call(url, method="GET", data=None):

    cmd = ["curl", "--silent", "-X", method,
           "https://api.cloudflare.com/client/v4%s" % url,
           "-H", "X-Auth-Email: %s" % CLOUDFLARE_EMAIL,
           "-H", "X-Auth-Key: %s" % CLOUDFLARE_AUTH_KEY,
           "-H", "Content-Type: application/json"
           ]
    if data:
        cmd.append("--data")
        cmd.append(data)
    try:
        log.debug("Executing: %r" % cmd)
        cf_process = Popen(cmd, stdout=PIPE, stderr=PIPE)
        cout, cerr = cf_process.communicate()

        if cf_process.returncode != 0:
            log.error(cerr)
            return None

        try:
            log.debug(cout)
            return json.loads(cout)

        except ValueError as err:
            log.error(err)
            log.error(cerr)
            return None

    except OSError as err:
        log.error(err)
        return None


def get_zone_id(domain_name):

    response = cf_api_call("/zones?name=%s" % domain_name)
    zone_id = response["result"][0]["id"]

    return zone_id


def get_record_id(domain_name, zone_id):

    url = "/zones/%s/dns_records?name=%s" % (zone_id, domain_name)

    response = cf_api_call(url)
    record_id = response["result"][0]["id"]

    return record_id


def update_dns_record(name, zone, ip, record_type="A", ttl=1):

    zone_id = get_zone_id(zone)

    record_id = get_record_id(name, zone_id)

    url = "/zones/%s/dns_records/%s" % (zone_id, record_id)
    data = {
        "id": record_id,
        "name": name,
        "content": ip,
        "type": record_type,
        "ttl": ttl
    }

    response = cf_api_call(url, method="PUT", data=json.dumps(data))

    if not response["success"]:
        for error in response["errors"]:
            log.error("Error(%d): %s" % (error["code"], error["message"]))

    return bool(response["success"])


def create_dns_record(name, zone, content, data=None, record_type="A", ttl=1):

    zone_id = get_zone_id(zone)

    url = "/zones/%s/dns_records" % zone_id
    request = {
        "name": name,
        "content": content,
        "type": record_type,
        "ttl": ttl
    }

    if data:
        request["data"] = data

    response = cf_api_call(url, method="POST", data=json.dumps(request))

    if not response["success"]:
        for error in response["errors"]:
            log.error("Error(%d): %s" % (error["code"], error["message"]))

    return bool(response["success"])


def delete_dns_record(name, zone):

    zone_id = get_zone_id(zone)
    record_id = get_record_id(name, zone_id)

    url = "/zones/%s/dns_records/%s" % (zone_id, record_id)

    response = cf_api_call(url, method="DELETE")

    if not response["success"]:
        for error in response["errors"]:
            log.error("Error(%d): %s" % (error["code"], error["message"]))

    return bool(response["success"])



import os
from twindb_infrastructure.config import TWINDB_INFRA_CONFIG
from twindb_infrastructure.config.credentials import Credentials, \
    CredentialsException


class AwsCredentials(Credentials):

    aws_access_key_id = None
    aws_secret_access_key = None
    aws_default_region = "us-east-1"
    aws_default_ami = "ami-6d1c2007"

    aws_instance_username = {
        aws_default_ami: "centos"
    }

    def __init__(self, config_path=TWINDB_INFRA_CONFIG):
        super(AwsCredentials, self).__init__(config_path=config_path)
        if not self.config.has_section('aws'):
            raise CredentialsException('There is no aws section in config %s'
                                       % self.config_path)
        for option in self.config.options('aws'):
            value = self.config.get('aws', option).strip('"\'')
            option_lower = option.lower()

            if option_lower.startswith('aws_instance_username_'):
                ami = option_lower.replace('aws_instance_username_', '')
                self.aws_instance_username[ami] = value
            else:
                setattr(self, option, value)
        os.environ["AWS_ACCESS_KEY_ID"] = self.aws_access_key_id
        os.environ["AWS_SECRET_ACCESS_KEY"] = self.aws_secret_access_key
        os.environ["AWS_DEFAULT_REGION"] = self.aws_default_region



import click
from subprocess import Popen
import json
import os
from twindb_infrastructure import setup_logging, log, __version__
from twindb_infrastructure.config import TWINDB_INFRA_CONFIG
from twindb_infrastructure.config.config import ConfigException
from twindb_infrastructure.util import parse_config


CONFIG = None


@click.group(invoke_without_command=True)
@click.option('--config', default=TWINDB_INFRA_CONFIG,
              help='Config file',
              show_default=True,
              type=click.Path())
@click.option('--debug', is_flag=True, default=False,
              help='Print debug messages')
@click.option('--version', help='Show tool version and exit.', is_flag=True,
              default=False)
@click.pass_context
def main(ctx, config, debug, version):
    """
    Console script to work with Chef
    """
    global CONFIG

    if not ctx.invoked_subcommand:
        if version:
            print(__version__)
        else:
            print(ctx.get_help())
        return 0

    setup_logging(log, debug=debug)
    log.debug('Using config %s' % config)
    try:
        CONFIG = parse_config(config)
    except ConfigException as err:  # pragma: no cover
        log.error(err)
        exit(-1)


def get_mounts_from_template(instance_profile):
    mounts = {}
    if not instance_profile \
            or "BlockDeviceMappings" not in instance_profile:
        return mounts

    for volume in instance_profile['BlockDeviceMappings']:
        key = volume['MountPoint'].replace('/', '_').lstrip('_')
        mounts[key] = {
            "device": volume['DeviceName'],
            "type": "xfs",
            "opts": "rw,relatime,attr2,inode64,noquota,noatime",
            "mount_point": volume['MountPoint']
        }
    return mounts


@main.command()
@click.option('--founder', default=False, is_flag=True,
              help='The node is a cluster founder')
@click.option('--recipe',
              help='Chef recipe to run')
@click.option('--cluster-name', help='Galer cluster name',
              default='PXC', show_default=True)
@click.option('--template', help='Instance profile')
@click.argument('ip')
@click.argument('node_name', required=False)
def bootstrap(founder, recipe, cluster_name, template, ip, node_name):
    """Run chef client on the server"""
    log.debug('node_name = %s' % node_name)
    node = None

    if node_name:
        node = node_name
    elif template:
        with open(template) as fp:
            instance_profile = json.loads(fp.read())
            node = instance_profile["Name"]
    if not node:
        log.error("If node_name agrument is not given "
                  "you must specify --template.")
        exit(1)

    cmd = [
        'knife',
        'bootstrap', ip, '--node-name', node,
        '--yes',
        '--ssh-user', 'centos', '--sudo',
        '--ssh-identity-file',
        os.path.expanduser('~/.ssh/id_rsa'),
        '--secret-file',
        os.path.expanduser('~/.chef/encrypted_data_bag_secret'),
        '--environment', 'staging',
        '--run-list', "role[base],recipe[%s]" % recipe
        ]
    attributes = {
        "percona": {
            "cluster": {
                "wsrep_cluster_name": cluster_name,
                "founder": founder
            }
        }
    }
    if template:
        with open(template) as fp:
            instance_profile = json.loads(fp.read())
            mounts = get_mounts_from_template(instance_profile)

            if mounts:
                attributes["fb_fstab"] = {
                    "mounts": mounts
                }

    cmd.append('--json-attributes')
    cmd.append(json.dumps(attributes))
    log.debug('Executing %s' % ' '.join(cmd))
    proc = Popen(cmd)
    proc.communicate()



import pytest
from click.testing import CliRunner

from twindb_infrastructure import twindb_chef
from twindb_infrastructure.twindb_chef import get_mounts_from_template


def test_main():
    runner = CliRunner()
    result = runner.invoke(twindb_chef.main)
    assert result.exit_code == 0


@pytest.mark.parametrize('instance_profile, mounts', [
    (
        None,
        {}
    ),
    (
        {},
        {}
    ),
    (
        {
            "ImageId": "ami-9b19f7fb",
            "InstanceType": "m4.4xlarge",
            "KeyName": "deployer",
            "SecurityGroupId": "sg-8dd1cae9",
            "SubnetId": "subnet-646cef3d",
            "RootVolumeSize": 200,
            "EbsOptimized": True,
            "UserName": "centos",
            "Name": "db-node01",
            "Region": "us-west-2"
        },
        {}
    ),
    (
        {
            "ImageId": "ami-9b19f7fb",
            "BlockDeviceMappings": [
                {
                    "DeviceName": "/dev/xvdb",
                    "VolumeSize": 200,
                    "VolumeType": "io1",
                    "MountPoint": "/var/lib/mysql",
                    "Iops": 4000
                },
                {
                    "DeviceName": "/dev/xvdc",
                    "VolumeSize": 300,
                    "VolumeType": "gp2",
                    "MountPoint": "/var/log/mysql"
                }
            ],
            "InstanceType": "m4.4xlarge",
            "KeyName": "deployer",
            "SecurityGroupId": "sg-8dd1cae9",
            "SubnetId": "subnet-646cef3d",
            "RootVolumeSize": 200,
            "EbsOptimized": True,
            "UserName": "centos",
            "Name": "db-node01",
            "Region": "us-west-2"
        },
        {
            "var_lib_mysql": {
                "device": "/dev/xvdb",
                "type": "xfs",
                "opts": "rw,relatime,attr2,inode64,noquota,noatime",
                "mount_point": "/var/lib/mysql"
            },
            "var_log_mysql": {
                "device": "/dev/xvdc",
                "type": "xfs",
                "opts": "rw,relatime,attr2,inode64,noquota,noatime",
                "mount_point": "/var/log/mysql"
            }
        }
    )


])
def test_get_mounts_from_tempate(instance_profile, mounts):
    assert get_mounts_from_template(instance_profile) == mounts



import mock
import pytest
from twindb_infrastructure.config.aws_credentials import AwsCredentials
from twindb_infrastructure.config.cloudflare_credentials import \
    CloudFlareCredentials
from twindb_infrastructure.config.config import Config, ConfigException
from twindb_infrastructure.config.credentials import Credentials, \
    CredentialsException
from twindb_infrastructure.config.ssh_credentials import SshCredentials


@pytest.fixture
def config(tmpdir):
    fp = tmpdir.join('config')
    fp.write('# this is a test config')
    return fp


@pytest.fixture
def private_key(tmpdir):
    fp = tmpdir.join('id_rsa')
    fp.write("""-----BEGIN RSA PRIVATE KEY-----
MIIEogIBAAKCAQEAqDshVjmu8khTpde8xVy2W4bD4BcZ4IzL4/AoCUJvxVtFT1xL
J7kG/8Zos/5DCM5Lq4+Q1T3umdoT7xNDJ8vOOoHkhXlB1IGnegqDciGnZASTxy0e
HmLZDwrJ6sWWdhySlCz2gUTlWembhHgQ7pFIlAKuUgmut2TP/tDjpNT2prEiarZt
UEs8AcpD6rDVO8ur3uNPYo5KTvDbe747BYz6B8Ksz9Pz3285F1Fes/OLQ1Pe6V/F
z8nmkewT9jGmUMwSQt33YXOlwy2jbKvd/JkhYWsRqyyK299YP57gPitfPrJlavxt
8E5Tw6/Ify2TCh9CfD/SKU43IAwe6UfCQDicDwIDAQABAoIBAF1z64LwrESe9Qit
nYmthQe3B1yWuKNK04Cdyj/Kjjh+CTSauo0odcDkQQmr9yUPJG37ZKNwsfj5chVf
B/E1gCx0N7QFthIMrDJZYMen9krTWBxO7epHUvjOL3ihpoGaQbrad108HoQiB2hB
InbEciL2kku0YUAzWm/dLnhEbXgPC894+20jWfeHOAGFgciGmgXHUZRmIImXJn+Y
/sfY/tkr+f8rD8nxLTsNJRWZJGNBifHjaIYjtKK7tz6SINSuc42Bx09cViSRw++N
wDVCM3ijXKGQi71lWYKDPNyhgXLAjngrcn0sRimhZcRMjvYpMKs0n34MVjDFWbUl
OaroPpECgYEA0jpcwryqG4RNpcIUxJnZ+CrbW46PrL/7G5hJJVZiEtLhFJJ5Wug2
6tVLKDk6P7YqktePp/k4kub7hOJ6p/bQUyRexVznYV/ynxMQ0AkDQqNP1sMNXQTm
cWi9a1OaD5h2Jg/f1swEpOUN3ZU6/GVAx8IxPkiijxkU8vSQ1zI6KV0CgYEAzNvy
7N8o3HVSUVaERjNfkCXTDLVohZ/IQM3t9XVlVSQDXgW19kyCTNmV0MCFGuqoIOJd
6sa+l5eHiC8PFASQ51PJ4aoQql7fF/k63G0pmm4CGoL/AaNhF7m4uAPEXIs7sL7o
Xhhpqx2HL9sYGe/XVEnDkkw68uh+LdZCntQ1CFsCgYA0eQjKOpkjAST4aLcSU2yK
evgBpFXMQqcEvkATp7oXBLfVkLHltOxwNQjrY01ctbVurYtX2+b5E9pX0sfWwM5C
0mMtVAEU1wQSHIonwvgjW+wDqO/e9egnCoOtFFLx7ZYf7fpq/MVz+xA47JSqhwNi
WOA9sZeRrCsEcXLto8XMqQKBgB2y4VPfwk+7nnR2Q3Td68O9CAy2m/GLSX/DmvTT
0R33u5k84LVVQCqd/K8nyeQuErO5vX3U1Dqr2BUxJVF0nAE9T24stn/MgzE5i4P7
O2XM/vcS+J8nLWNAJHxg5223La8g7hT+GwuYm0mfzK2t7JymmPiznnQqqmhAwKXA
A/QzAoGAGQAzjHkhlUEHBSGEuzPG5C2hxlu0/pPysti2NIlwWG6vy2coyfAPDK0a
ArKlXxiiEsDbyRolRDMLfq4ovCFFqv3q+fY80/owBu3KcaXN260IQTpHuVRYM9Ax
XFS1IpIg+KIzUDtmGDi5uIB4KTps3YmpF3iqCn6oQkQnFZBFBbY=
-----END RSA PRIVATE KEY-----
""")
    fp.chmod(0400)
    return fp


@mock.patch.object(AwsCredentials, '__init__')
@mock.patch.object(SshCredentials, '__init__')
@mock.patch.object(CloudFlareCredentials, '__init__')
def test_config_init(mock_cf, mock_ssh, mock_aws, config):
    mock_aws.return_value = None
    mock_ssh.return_value = None
    mock_cf.return_value = None

    c = Config(str(config))
    c.config_path = str(config)
    mock_aws.assert_called_once_with(config_path=str(config))
    mock_ssh.assert_called_once_with(config_path=str(config))
    mock_cf.assert_called_once_with(config_path=str(config))


@mock.patch.object(AwsCredentials, '__init__')
@mock.patch.object(SshCredentials, '__init__')
@mock.patch.object(CloudFlareCredentials, '__init__')
def test_config_init_credentials_exception(mock_cf, mock_ssh, mock_aws, config):
    mock_aws.return_value = None
    mock_ssh.return_value = None
    mock_cf.side_effect = CredentialsException

    with pytest.raises(ConfigException):
        Config(str(config))


def test_credentials_raises_exception():
    with pytest.raises(CredentialsException):
        Credentials('aaa')


def test_credentials_init(config):
    config.write('[foo]')
    c = Credentials(str(config))
    assert c.config_path == str(config)


def test_malformed_config(config):
    config.write('aaa')
    with pytest.raises(CredentialsException):
        Credentials(str(config))


def test_empty_config(config):
    with pytest.raises(CredentialsException):
        Credentials(str(config))


def test_aws_credentials_raises_exception_if_no_aws_section(tmpdir):
    with pytest.raises(CredentialsException):
        config_file = tmpdir.join('some_config')
        config_file.write('[foo]')
        AwsCredentials(config_path=str(config_file))


def test_aws_sets_options(config):
    config.write("""
[aws]
AWS_ACCESS_KEY_ID = "foo"
AWS_SECRET_ACCESS_KEY = 'bar'
AWS_DEFAULT_REGION = 'xxx'
AWS_DEFAULT_AMI = 'yyy'
AWS_INSTANCE_USERNAME_yyy = "centos"
""")
    ac = AwsCredentials(config_path=str(config))
    assert ac.aws_access_key_id == 'foo'
    assert ac.aws_secret_access_key == 'bar'
    assert ac.aws_default_region == 'xxx'
    assert ac.aws_default_ami == 'yyy'
    assert ac.aws_instance_username['yyy'] == 'centos'


def test_cloudflare_sets_options(config):
    config.write("""
[cloudflare]
CLOUDFLARE_EMAIL = "foo"
CLOUDFLARE_AUTH_KEY = "bar"
""")
    cfc = CloudFlareCredentials(config_path=str(config))
    assert cfc.cloudflare_email == 'foo'
    assert cfc.cloudflare_auth_key == 'bar'


def test_cloudflare_no_exception(config):
    config.write('[foo]')
    CloudFlareCredentials(config_path=str(config))


def test_ssh_sets_options(config, private_key):
    config.write("""
[ssh]
private_key_file = %s
""" % str(private_key))
    c = SshCredentials(config_path=str(config))
    assert c.private_key == """-----BEGIN RSA PRIVATE KEY-----
MIIEogIBAAKCAQEAqDshVjmu8khTpde8xVy2W4bD4BcZ4IzL4/AoCUJvxVtFT1xL
J7kG/8Zos/5DCM5Lq4+Q1T3umdoT7xNDJ8vOOoHkhXlB1IGnegqDciGnZASTxy0e
HmLZDwrJ6sWWdhySlCz2gUTlWembhHgQ7pFIlAKuUgmut2TP/tDjpNT2prEiarZt
UEs8AcpD6rDVO8ur3uNPYo5KTvDbe747BYz6B8Ksz9Pz3285F1Fes/OLQ1Pe6V/F
z8nmkewT9jGmUMwSQt33YXOlwy2jbKvd/JkhYWsRqyyK299YP57gPitfPrJlavxt
8E5Tw6/Ify2TCh9CfD/SKU43IAwe6UfCQDicDwIDAQABAoIBAF1z64LwrESe9Qit
nYmthQe3B1yWuKNK04Cdyj/Kjjh+CTSauo0odcDkQQmr9yUPJG37ZKNwsfj5chVf
B/E1gCx0N7QFthIMrDJZYMen9krTWBxO7epHUvjOL3ihpoGaQbrad108HoQiB2hB
InbEciL2kku0YUAzWm/dLnhEbXgPC894+20jWfeHOAGFgciGmgXHUZRmIImXJn+Y
/sfY/tkr+f8rD8nxLTsNJRWZJGNBifHjaIYjtKK7tz6SINSuc42Bx09cViSRw++N
wDVCM3ijXKGQi71lWYKDPNyhgXLAjngrcn0sRimhZcRMjvYpMKs0n34MVjDFWbUl
OaroPpECgYEA0jpcwryqG4RNpcIUxJnZ+CrbW46PrL/7G5hJJVZiEtLhFJJ5Wug2
6tVLKDk6P7YqktePp/k4kub7hOJ6p/bQUyRexVznYV/ynxMQ0AkDQqNP1sMNXQTm
cWi9a1OaD5h2Jg/f1swEpOUN3ZU6/GVAx8IxPkiijxkU8vSQ1zI6KV0CgYEAzNvy
7N8o3HVSUVaERjNfkCXTDLVohZ/IQM3t9XVlVSQDXgW19kyCTNmV0MCFGuqoIOJd
6sa+l5eHiC8PFASQ51PJ4aoQql7fF/k63G0pmm4CGoL/AaNhF7m4uAPEXIs7sL7o
Xhhpqx2HL9sYGe/XVEnDkkw68uh+LdZCntQ1CFsCgYA0eQjKOpkjAST4aLcSU2yK
evgBpFXMQqcEvkATp7oXBLfVkLHltOxwNQjrY01ctbVurYtX2+b5E9pX0sfWwM5C
0mMtVAEU1wQSHIonwvgjW+wDqO/e9egnCoOtFFLx7ZYf7fpq/MVz+xA47JSqhwNi
WOA9sZeRrCsEcXLto8XMqQKBgB2y4VPfwk+7nnR2Q3Td68O9CAy2m/GLSX/DmvTT
0R33u5k84LVVQCqd/K8nyeQuErO5vX3U1Dqr2BUxJVF0nAE9T24stn/MgzE5i4P7
O2XM/vcS+J8nLWNAJHxg5223La8g7hT+GwuYm0mfzK2t7JymmPiznnQqqmhAwKXA
A/QzAoGAGQAzjHkhlUEHBSGEuzPG5C2hxlu0/pPysti2NIlwWG6vy2coyfAPDK0a
ArKlXxiiEsDbyRolRDMLfq4ovCFFqv3q+fY80/owBu3KcaXN260IQTpHuVRYM9Ax
XFS1IpIg+KIzUDtmGDi5uIB4KTps3YmpF3iqCn6oQkQnFZBFBbY=
-----END RSA PRIVATE KEY-----
"""
    assert c.public_key == """ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQCoOyFWOa7ySFOl17zFXLZbhsPgFxngjMvj8CgJQm/FW0VPXEsnuQb/xmiz/kMIzkurj5DVPe6Z2hPvE0Mny846geSFeUHUgad6CoNyIadkBJPHLR4eYtkPCsnqxZZ2HJKULPaBROVZ6ZuEeBDukUiUAq5SCa63ZM/+0OOk1PamsSJqtm1QSzwBykPqsNU7y6ve409ijkpO8Nt7vjsFjPoHwqzP0/PfbzkXUV6z84tDU97pX8XPyeaR7BP2MaZQzBJC3fdhc6XDLaNsq938mSFhaxGrLIrb31g/nuA+K18+smVq/G3wTlPDr8h/LZMKH0J8P9IpTjcgDB7pR8JAOJwP
"""


def test_ssh_raises_exception(config):
    config.write('[foo]')
    with pytest.raises(CredentialsException):
        SshCredentials(config_path=str(config))



from subprocess import Popen, PIPE
from twindb_infrastructure.config import TWINDB_INFRA_CONFIG
from twindb_infrastructure.config.credentials import Credentials, \
    CredentialsException


class SshCredentials(Credentials):

    private_key_file = None
    private_key = None
    public_key = None

    def __init__(self, config_path=TWINDB_INFRA_CONFIG):
        super(SshCredentials, self).__init__(config_path=config_path)
        my_section = 'ssh'
        if not self.config.has_section(my_section):
            raise CredentialsException('There is no %s '
                                       'section in config %s'
                                       % (my_section, self.config_path))
        self.private_key_file = self.config.get(my_section,
                                                'private_key_file')\
            .strip('"\'')
        with open(self.private_key_file) as fp:
            self.private_key = fp.read()

        p = Popen(['ssh-keygen', '-y', '-f', self.private_key_file],
                  stdout=PIPE, stderr=PIPE)
        self.public_key, cerr = p.communicate()



import sys
from subprocess import Popen
from twindb_infrastructure import log
from twindb_infrastructure.config.config import Config

CONFIG = None


def printf(fmt, *args):
    sys.stdout.write(fmt % args)


def parse_config(path):
    """Parse TwinDB Infrastructure config

    :param path: path to config file
    :raise ConfigException if config can't be parsed
    """
    return Config(path)


def _execute_remote(cmd, node):

    ssh_cmd = ['ssh', '-t', '-o', 'StrictHostKeyChecking=no', '-i',
               '/home/twindbcom/.ssh/id_rsa', '-l', 'centos', node, cmd]
    log.debug('Executing %s' % ' '.join(ssh_cmd))
    proc = Popen(ssh_cmd)
    proc.communicate()


def stop_chef_client(nodes):
    log.info('Stopping Chef client on %s' % ', '.join(nodes))
    for node in nodes:
        _execute_remote("sudo systemctl stop chef-client", node)


def start_chef_client(nodes):
    log.info('Starting Chef client on %s' % ', '.join(nodes))
    for node in nodes:
        _execute_remote("sudo systemctl start chef-client", node)


def stop_galera(nodes):
    log.info('Stopping Galera on %s' % ', '.join(nodes))
    for node in nodes:
        _execute_remote("sudo systemctl stop mysql.service", node)
        _execute_remote("sudo systemctl stop mysql@bootstrap.service", node)


def start_galera(nodes):
    log.info('Starting Galera on %s' % ', '.join(nodes))
    for node in nodes:
        _execute_remote("sudo systemctl start mysql", node)


def remote_rmdir(directory, nodes):
    for node in nodes:
        log.info('Removing %s on %s' % (directory, node))
        _execute_remote("sudo rm -rf \"%s\"/*" % directory, node)


def remote_restore(node, backup_copy, datadir):
    log.info('Restoring backup copy %s on %s' % (backup_copy, node))
    _execute_remote("sudo twindb-backup restore mysql --dst %s %s"
                    % (datadir, backup_copy),
                    node)


def bootstrap_first_node(node, datadir):
    _execute_remote("sudo chown -R mysql:mysql '%s'" % (datadir, ), node)
    _execute_remote("sudo systemctl start mysql@bootstrap.service", node)


def bootstrap_next_node(node, datadir):

    _execute_remote("sudo mysql_install_db", node)
    _execute_remote('sudo chown -R mysql:mysql "%s"' % (datadir, ), node)
    _execute_remote("sudo systemctl start mysql", node)


def domainname(name):
    """Extracts domain name from a fqdn

    :param name: FQDN like www.google.com or www.yahoo.com.
    :type name: str
    :return: domain name like google.com  or yahoo.com.
    :rtype: str
    """
    result = name.split('.')
    result.pop(0)
    return '.'.join(result)
